@@ -272,6 +272,7 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
     const char *cur = URL;
     char buf[4096];
     int indx = 0;
+    const int indxMax = 4096 - 1;
     int port = 0;
 
     if (ctxt->protocol != NULL) { 
@@ -288,7 +289,7 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
     }
     if (URL == NULL) return;
     buf[indx] = 0;
-    while (*cur != 0) {
+    while ((*cur != 0) && (indx < indxMax)) {
         if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
 	    buf[indx] = 0;
 	    ctxt->protocol = xmlMemStrdup(buf);
@@ -301,7 +302,7 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
     if (*cur == 0) return;
 
     buf[indx] = 0;
-    while (1) {
+    while (indx < indxMax) {
 	if ((strchr (cur, '[') && !strchr (cur, ']')) ||
 		(!strchr (cur, '[') && strchr (cur, ']'))) {
 	    __xmlIOErr(XML_FROM_HTTP, XML_HTTP_URL_SYNTAX, 
@@ -311,7 +312,7 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
 
 	if (cur[0] == '[') {
 	    cur++;
-	    while (cur[0] != ']')
+	    while ((cur[0] != ']') && (indx < indxMax))
 		buf[indx++] = *cur++;
     
 	    if (!strchr (buf, ':')) {
@@ -368,7 +369,7 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
     else {
         indx = 0;
         buf[indx] = 0;
-	while (*cur != 0)
+	while ((*cur != 0) && (indx < indxMax))
 	    buf[indx++] = *cur++;
 	buf[indx] = 0;
 	ctxt->path = xmlMemStrdup(buf);
@@ -390,6 +391,7 @@ xmlNanoHTTPScanProxy(const char *URL) {
     const char *cur = URL;
     char buf[4096];
     int indx = 0;
+    const int indxMax = 4096 - 1;
     int port = 0;
 
     if (proxy != NULL) { 
@@ -409,7 +411,7 @@ xmlNanoHTTPScanProxy(const char *URL) {
 #endif
     if (URL == NULL) return;
     buf[indx] = 0;
-    while (*cur != 0) {
+    while ((*cur != 0) && (indx < indxMax)) {
         if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
 	    buf[indx] = 0;
 	    indx = 0;
@@ -421,7 +423,7 @@ xmlNanoHTTPScanProxy(const char *URL) {
     if (*cur == 0) return;
 
     buf[indx] = 0;
-    while (1) {
+    while (indx < indxMax) {
 	if ((strchr (cur, '[') && !strchr (cur, ']')) ||
 		(!strchr (cur, '[') && strchr (cur, ']'))) {
 	    __xmlIOErr(XML_FROM_HTTP, XML_HTTP_URL_SYNTAX, "Syntax Error\n");
@@ -430,7 +432,7 @@ xmlNanoHTTPScanProxy(const char *URL) {
 
 	if (cur[0] == '[') {
 	    cur++;
-	    while (cur[0] != ']')
+	    while ((cur[0] != ']') && (indx < indxMax))
 		buf[indx++] = *cur++;
 
 	    if (!strchr (buf, ':')) {
@@ -872,6 +874,9 @@ static int
 xmlNanoHTTPConnectAttempt(struct sockaddr *addr)
 {
     fd_set wfd;
+#ifdef _WINSOCKAPI_
+    fd_set xfd;
+#endif
     struct timeval tv;
     int status;
     int addrlen;
@@ -909,6 +914,12 @@ xmlNanoHTTPConnectAttempt(struct sockaddr *addr)
 	status = ioctl(s, FIONBIO, &enable);
     }
 #else /* VMS */
+#if defined(__BEOS__)
+	{
+		bool noblock = true;
+		status = setsockopt(s, SOL_SOCKET, SO_NONBLOCK, &noblock, sizeof(noblock));
+	}
+#else /* __BEOS__ */
     if ((status = fcntl(s, F_GETFL, 0)) != -1) {
 #ifdef O_NONBLOCK
 	status |= O_NONBLOCK;
@@ -927,6 +938,7 @@ xmlNanoHTTPConnectAttempt(struct sockaddr *addr)
 	closesocket(s);
 	return(-1);
     }
+#endif /* !__BEOS__ */
 #endif /* !VMS */
 #endif /* !_WINSOCKAPI_ */
 
@@ -947,8 +959,15 @@ xmlNanoHTTPConnectAttempt(struct sockaddr *addr)
     
     FD_ZERO(&wfd);
     FD_SET(s, &wfd);
+
+#ifdef _WINSOCKAPI_    
+    FD_ZERO(&xfd);
+    FD_SET(s, &xfd);
     
+    switch(select(s+1, NULL, &wfd, &xfd, &tv))
+#else
     switch(select(s+1, NULL, &wfd, NULL, &tv))
+#endif
     {
 	case 0:
 	    /* Time out */
@@ -962,7 +981,11 @@ xmlNanoHTTPConnectAttempt(struct sockaddr *addr)
 	    return(-1);
     }
 
-    if ( FD_ISSET(s, &wfd) ) {
+    if ( FD_ISSET(s, &wfd)
+#ifdef _WINSOCKAPI_
+                           || FD_ISSET(s, &xfd)
+#endif
+                                                ) {
 	SOCKLEN_T len;
 	len = sizeof(status);
 #ifdef SO_ERROR
@@ -1314,7 +1337,7 @@ retry:
     if (contentType && *contentType)
 	blen += strlen(*contentType) + 16;
     blen += strlen(method) + strlen(ctxt->path) + 24;
-    bp = xmlMallocAtomic(blen);
+    bp = (char*)xmlMallocAtomic(blen);
     if ( bp == NULL ) {
         xmlNanoHTTPFreeCtxt( ctxt );
 	xmlHTTPErrMemory("allocating header buffer");
@@ -1616,7 +1639,7 @@ xmlNanoHTTPAuthHeader(void *ctx) {
  */
 int
 xmlNanoHTTPContentLength( void * ctx ) {
-    xmlNanoHTTPCtxtPtr	ctxt = ctx;
+    xmlNanoHTTPCtxtPtr	ctxt = (xmlNanoHTTPCtxtPtr)ctx;
 
     return ( ( ctxt == NULL ) ? -1 : ctxt->ContentLength );
 }
@@ -1631,7 +1654,7 @@ xmlNanoHTTPContentLength( void * ctx ) {
  */
 const char *
 xmlNanoHTTPRedir( void * ctx ) {
-    xmlNanoHTTPCtxtPtr	ctxt = ctx;
+    xmlNanoHTTPCtxtPtr	ctxt = (xmlNanoHTTPCtxtPtr)ctx;
 
     return ( ( ctxt == NULL ) ? NULL : ctxt->location );
 }
@@ -1646,7 +1669,7 @@ xmlNanoHTTPRedir( void * ctx ) {
  */
 const char *
 xmlNanoHTTPEncoding( void * ctx ) {
-    xmlNanoHTTPCtxtPtr	ctxt = ctx;
+    xmlNanoHTTPCtxtPtr	ctxt = (xmlNanoHTTPCtxtPtr)ctx;
 
     return ( ( ctxt == NULL ) ? NULL : ctxt->encoding );
 }
@@ -1661,7 +1684,7 @@ xmlNanoHTTPEncoding( void * ctx ) {
  */
 const char *
 xmlNanoHTTPMimeType( void * ctx ) {
-    xmlNanoHTTPCtxtPtr	ctxt = ctx;
+    xmlNanoHTTPCtxtPtr	ctxt = (xmlNanoHTTPCtxtPtr)ctx;
 
     return ( ( ctxt == NULL ) ? NULL : ctxt->mimeType );
 }
@@ -1680,7 +1703,7 @@ xmlNanoHTTPMimeType( void * ctx ) {
  */
 int
 xmlNanoHTTPFetchContent( void * ctx, char ** ptr, int * len ) {
-    xmlNanoHTTPCtxtPtr	ctxt = ctx;
+    xmlNanoHTTPCtxtPtr	ctxt = (xmlNanoHTTPCtxtPtr)ctx;
 
     int			rc = 0;
     int			cur_lgth;
